<?php

class Virtuemart
{
  public function validatePassword($hash, $password, $salt)
  {
    return $this->validateV1($hash, $password, $salt) || $this->validateV2($hash, $password, $salt);
  }

  public static function validateV1($hash, $password, $salt)
  {
    $encrypted = ($salt) ? md5($password.$salt) : md5($password);
    return $hash == $encrypted;
  }

  public static function validateV2($hash, $password, $salt)
  {
    if (strpos($hash, '$P$') === 0) {
      return PasswordHash::init(10, true)->CheckPassword($password, $hash);
    } elseif ($hash[0] == '$') {
      return self::passwordVerify($password, $hash);
    } elseif (substr($hash, 0, 8) == '{SHA256}') {
      $parts     = explode(':', $hash);
      $salt      = @$parts[1];

      $encrypted = ($salt) ? hash('sha256', $password . $salt) . ':' . $salt : hash('sha256', $password);
      $testcrypt = '{SHA256}' . $encrypted;

      return self::_timingSafeCompare($hash, $testcrypt);
    }

    $parts = explode(':', $hash);
    $salt  = @$parts[1];

    $testcrypt = md5($password . $salt) . ($salt ? ':' . $salt : (strpos($hash, ':') !== false ? ':' : ''));

    return self::_timingSafeCompare($hash, $testcrypt);
  }

  private static function passwordVerify($password, $hash)
  {
    if (!function_exists('crypt')) {
      return false;
    }

    $ret = crypt($password, $hash);
    if (!is_string($ret) || self::_strlen($ret) != self::_strlen($hash) || self::_strlen($ret) <= 13) {
      return false;
    }

    $status = 0;
    for ($i = 0; $i < self::_strlen($ret); $i++) {
      $status |= (ord($ret[$i]) ^ ord($hash[$i]));
    }

    return $status === 0;
  }

  private static function _strlen($string) {
    if (function_exists('mb_strlen')) {
      return mb_strlen($string, '8bit');
    }
    return strlen($string);
  }

  private static function _timingSafeCompare($known, $unknown)
  {
    $known .= chr(0);
    $unknown .= chr(0);

    $knownLength = strlen($known);
    $unknownLength = strlen($unknown);

    $result = $knownLength - $unknownLength;

    for ($i = 0; $i < $unknownLength; $i++) {
      $result |= (ord($known[$i % $knownLength]) ^ ord($unknown[$i]));
    }

    return $result === 0;
  }
}